package de.duehl.vocabulary.japanese.ui.dialog;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Point;
import java.util.List;

import javax.swing.JPanel;
import javax.swing.JScrollPane;

import de.duehl.basics.datetime.date.ImmutualDate;
import de.duehl.basics.text.Text;
import de.duehl.swing.ui.GuiTools;
import de.duehl.swing.ui.components.selections.SelectionsHelper;
import de.duehl.swing.ui.components.selections.StringSelection;
import de.duehl.swing.ui.components.selections.TextAreaSelection;
import de.duehl.swing.ui.dialogs.base.ModalDialogBase;
import de.duehl.swing.ui.layout.VerticalLayout;
import de.duehl.vocabulary.japanese.data.Vocable;
import de.duehl.vocabulary.japanese.logic.internal.InternalDataRequester;
import de.duehl.vocabulary.japanese.logic.internal.data.InternalAdditionalVocableData;
import de.duehl.vocabulary.japanese.logic.persistence.Options;
import de.duehl.vocabulary.japanese.ui.components.VocableViewer;

/**
 * Diese Klasse zeigt eine Vokabel mit den zugehörigen benutzerabhängigen Daten an.
 *
 * @version 1.01     2024-04-22
 * @author Christian Dühl
 */

public class VocableWithInternaViewer extends ModalDialogBase {

    private static final Dimension DIALOG_DIMENSION = new Dimension(1000, 600);


    /** Die Programmoptionen. */
    private final Options options;

    /** Das Objekt das zu einer Vokabel die internen, benutzerabhängigen Daten abrufen kann. */
    private final InternalDataRequester requester;

    /** Die anzuzeigende Vokabel. */
    private final Vocable vocable;

    /** Die internen Daten zur übergebenen Vokabel. */
    private final InternalAdditionalVocableData data;

    /**
     * Konstruktor.
     *
     * @param options
     *            Die Programmoptionen.
     * @param requester
     *            Das Objekt das zu einer Vokabel die internen, benutzerabhängigen Daten abrufen
     *            kann.
     * @param vocable
     *            Die anzuzeigende Vokabel.
     * @param parentLocation
     *            Die Position des Rahmens der Oberfläche, vor der dieser Dialog erzeugt wird.
     * @param programImage
     *            Das anzuzeigende ProgrammIcon.
     */
    public VocableWithInternaViewer(Options options, InternalDataRequester requester,
            Vocable vocable, Point parentLocation, Image programImage) {
        super(parentLocation, programImage,
                "Ansicht einer Vokabel mit den zugehörigen benutzerabhängigen Daten",
                DIALOG_DIMENSION);
        addEscapeBehaviour();

        this.options = options;
        this.requester = requester;
        this.vocable = vocable;
        this.data = requester.getInternalDataForVocable(vocable);

        fillDialog();
    }

    /** Baut die Gui auf. */
    @Override
    protected void populateDialog() {
        JScrollPane scroll = GuiTools.createScrollPane(createMainPanel());
        add(scroll, BorderLayout.CENTER);

        GuiTools.scrollScrollbarToMinimumLater(scroll);
    }

    private Component createMainPanel() {
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());

        panel.add(createVocableViewerPart(), BorderLayout.NORTH);
        panel.add(createVocableAndInternalDataSectionPart(), BorderLayout.CENTER);

        return panel;
    }

    private Component createVocableViewerPart() {
        VocableViewer viewer = new VocableViewer(options);
        viewer.showVocable(vocable, requester.getInternalDataForVocable(vocable));
        return viewer.getPanel();
    }

    private Component createVocableAndInternalDataSectionPart() {
        JPanel panel = new JPanel();
        panel.setLayout(new VerticalLayout(0, VerticalLayout.BOTH));

        panel.add(createKanaPart());
        panel.add(createKanjiPart());
        panel.add(createRomajiPart());
        panel.add(createPronunciationPart());
        panel.add(createTranslationsPart());
        panel.add(createCommentPart());
        panel.add(createSearchWordsPart());
        panel.add(createPartsOfSpeechPart());
        panel.add(createVocabularyDescriptionPart());
        panel.add(createFirstSeenDatePart());
        panel.add(createJapaneseToGermanTestCountPart());
        panel.add(createCorrectJapaneseToGermanTestCountPart());
        panel.add(createLastJapaneseToGermanTestDatePart());
        panel.add(createLastCorrectJapaneseToGermanTestDatePart());
        panel.add(createLastTenJapaneseToGermanTestResultsPart());
        panel.add(createGermanToJapaneseTestCountPart());
        panel.add(createCorrectGermanToJapaneseTestCountPart());
        panel.add(createLastGermanToJapaneseTestDatePart());
        panel.add(createLastCorrectGermanToJapaneseTestDatePart());
        panel.add(createLastTenGermanToJapaneseTestResultsPart());

        return panel;
    }

    private Component createKanaPart() {
        return createSingleStringPart("Kana", vocable.getKana());
    }

    private Component createKanjiPart() {
        return createSingleStringPart("Kanji", vocable.getKanji());
    }

    private Component createRomajiPart() {
        return createSingleStringPart("Romaji", vocable.getRomaji());
    }

    private Component createPronunciationPart() {
        return createSingleStringPart("Aussprache", vocable.getPronunciation());
    }

    private Component createTranslationsPart() {
        List<String> translations = vocable.getTranslations();
        return createListPart("Übersetzung", "Übersetzungen", translations);
    }

    private Component createCommentPart() {
        return createSingleLongStringPart("Kommentar", vocable.getComment());
    }

    private Component createSearchWordsPart() {
        List<String> searchWords = vocable.getSearchWords();
        return createListPart("Suchbegriff", "Suchbegriffe", searchWords);
    }

    private Component createPartsOfSpeechPart() {
        List<String> partsOfSpeach = vocable.getPartsOfSpeech();
        return createListPart("Wortart", "Wortarten", partsOfSpeach);
    }

    private Component createVocabularyDescriptionPart() {
        return createSingleStringPart("Beschreibung des Vokabulars",
                vocable.getVocabularyDescription());
    }

    private Component createFirstSeenDatePart() {
        return createDatePart("Zuerst gesehen am", data.getFirstSeenDate());
    }

    private Component createJapaneseToGermanTestCountPart() {
        return createIntPart("Anzahl der Übersetzungen Japanisch - Deutsch",
                data.getJapaneseToGermanTestCount());
    }

    private Component createCorrectJapaneseToGermanTestCountPart() {
        return createIntPart("Anzahl der richtigen Übersetzungen Japanisch - Deutsch",
                data.getCorrectJapaneseToGermanTestCount());
    }

    private Component createLastJapaneseToGermanTestDatePart() {
        return createDatePart("Zuletzt abgefragt Japanisch - Deutsch",
                data.getLastJapaneseToGermanTestDate());
    }

    private Component createLastCorrectJapaneseToGermanTestDatePart() {
        return createDatePart("Zuletzt abgefragt und richtig beantwortet Japanisch - Deutsch",
                data.getLastCorrectJapaneseToGermanTestDate());
    }

    private Component createLastTenJapaneseToGermanTestResultsPart() {
        return createSingleStringPart("Erfolg der letzten 10 Abfragen Japanisch - Deutsch",
                data.getLastTenJapaneseToGermanTestResultsAsStorageString());
    }

    private Component createGermanToJapaneseTestCountPart() {
        return createIntPart("Anzahl der Übersetzungen Deutsch - Japanisch",
                data.getGermanToJapaneseTestCount());
    }

    private Component createCorrectGermanToJapaneseTestCountPart() {
        return createIntPart("Anzahl der richtigen Übersetzungen Deutsch - Japanisch",
                data.getCorrectGermanToJapaneseTestCount());
    }

    private Component createLastGermanToJapaneseTestDatePart() {
        return createDatePart("Zuletzt abgefragt Deutsch - Japanisch",
                data.getLastGermanToJapaneseTestDate());

    }

    private Component createLastCorrectGermanToJapaneseTestDatePart() {
        return createDatePart("Zuletzt abgefragt und richtig beantwortet Deutsch - Japanisch",
                data.getLastCorrectGermanToJapaneseTestDate());
    }

    private Component createLastTenGermanToJapaneseTestResultsPart() {
        return createSingleStringPart("Erfolg der letzten 10 Abfragen Deutsch - Japanisch",
                data.getLastTenGermanToJapaneseTestResultsAsStorageString());
    }

    private Component createSingleStringPart(String title, String value) {
        StringSelection selection = new StringSelection(title);
        SelectionsHelper.initSelectionAsViewer(selection);
        selection.setFocusable(true);
        selection.setText(value);
        return selection.getPanel();
    }

    private Component createSingleLongStringPart(String title, String value) {
        String brokenText = Text.addLineBreaks(value, 60);
        List<String> lines = Text.splitByLineBreaks(brokenText);
        int numberOfLines = lines.size();
        if (numberOfLines == 1) {
            return createSingleStringPart(title, lines.get(0));
        }
        else {
            TextAreaSelection selection = new TextAreaSelection(title);
            SelectionsHelper.initSelectionAsViewer(selection);
            selection.setFocusable(true);
            selection.setText(brokenText);
            selection.setRows(numberOfLines);
            return selection.getPanel();
        }
    }

    private Component createListPart(String titleForOneElement, String titleForMoreElements,
            List<String> values) {
        if (values.size() == 1) {
            return createSingleStringPart(titleForOneElement, values.get(0));
        }
        else {
            TextAreaSelection selection = new TextAreaSelection(titleForMoreElements);
            SelectionsHelper.initSelectionAsViewer(selection);
            selection.setFocusable(true);
            selection.setText(Text.joinWithLineBreak(values));
            return selection.getPanel();
        }
    }

    private Component createDatePart(String title, ImmutualDate date) {
        String value;
        if (date.equals(InternalAdditionalVocableData.NOT_SEEN_DATE)) {
            value = "";
        }
        else {
            value = date.toString();
        }
        return createSingleStringPart(title, value);
    }

    private Component createIntPart(String title, int value) {
        return createSingleStringPart(title, Integer.toString(value));
    }

}
