package de.duehl.vocabulary.japanese.ui.dialog.table.kanji;

import java.util.List;

import javax.swing.table.AbstractTableModel;

import de.duehl.basics.text.Text;
import de.duehl.vocabulary.japanese.data.symbol.Kanji;

/**
 * Diese Klasse ist ein eigenes Tabellenmodell für die Ansicht der (bislang eingepflegten) Kanji.
 * Es hält die Daten der Tabelle in Form einer Liste von Kanji-Objekten vor und weiß diese
 * anzuzeigen.
 *
 * @version 1.01     2024-10-02
 * @author Christian Dühl
 */

public class KanjiTableModel extends AbstractTableModel {

    private static final int NUMBER_OF_COLUMNS = 10;

    private static final long serialVersionUID = 1L;


    /** Die Liste mit den Kanji der Tabelle. */
    private final List<Kanji> kanjiList;

    /**
     * Konstruktor.
     *
     * @param kanjiList
     *            Die Liste mit den Kanji der Tabelle.
     */
    public KanjiTableModel(List<Kanji> kanjiList) {
        super();
        this.kanjiList = kanjiList;
    }

    /** Ermittelt die Zeilenzahl. */
    @Override
    public int getRowCount() {
        return kanjiList.size();
    }

    /** Ermittelt die Spaltenzahl. */
    @Override
    public int getColumnCount() {
        return NUMBER_OF_COLUMNS;
    }

    /**
     * Gibt an, ob die Zelle bearbeitbar ist. Dies muss für die Buttons er Fall sein.
     *
     * @param rowIndex
     *            Zeilenindex (die erste hat die 0)
     * @param columnIndex
     *            Spaltenindex (die erste hat die 0)
     */
    @Override
    public boolean isCellEditable(int rowIndex, int columnIndex) {
        return columnIndex == 9;
    }

    /**
     * Ermittelt den Tabelleninhalt der angegebenen Zelle.
     *
     * Die Tabelle enthält die folgenden Spalten:
     *     - Die laufende Nummer.
     *     - Das Kanji in UTF-8 Darstellung.
     *     - Die deutsche Beschreibung.
     *     - Das Wort in Kanji und Hiragana in der häufigsten Bedeutung.
     *     - Das Wort in Hiragana in der häufigsten Bedeutung.
     *     - Die Nummer des Kanji im Buch "Kanji und Kana" von Wolfgang Hadamitzky.
     *     - Die Seite auf der das Kanji im Buch "Kanji und Kana" von W. Hadamitzky zu finden ist.
     *     - Die Liste der sinojapanischen ON-Lesungen des Kanji (groß geschrieben).
     *     - Die Liste der rein japanischen kun-Lesungen des Kanji (klein geschrieben).
     *     - Button zum Betrachten der aller Vokabeln, in denen das Kanji vorkommt.
     *
     * @param rowIndex
     *            Zeilenindex (die erste hat die 0)
     * @param columnIndex
     *            Spaltenindex (die erste hat die 0)
     */
    @Override
    public Object getValueAt(int rowIndex, int columnIndex) {
        Kanji kanji = kanjiList.get(rowIndex);
        switch (columnIndex) {
            case 0:
                return rowIndex + 1;
            case 1:
                return kanji.getCharacter();
            case 2:
                return kanji.getGermanDescription();
            case 3:
                return kanji.getNormalKanjiHiraganaWord();
            case 4:
                return kanji.getNormalHiraganaWord();
            case 5:
                return kanji.getNumberInKanjiAndKanaBook();
            case 6:
                return kanji.getPageNumberInKanjiAndKanaBook();
            case 7:
                return Text.join(", ", kanji.getOnLesungen());
            case 8:
                return Text.join(", ", kanji.getKunLesungen());
            case 9:
                return "Vokabeln anzeigen";
            default:
                throw new RuntimeException("Unzuläsiger Spaltenindex '" + columnIndex + "'.");
        }
    }

    @Override
    public Class<?> getColumnClass(int columnIndex) {
        switch (columnIndex) {
            case 0:
            case 5:
            case 6:
                return Integer.class;
            case 1:
            case 2:
            case 3:
            case 4:
            case 7:
            case 8:
                return String.class;
                // 9 ist der Button zum Anzeigen der zugehörigen Vokabeln
            default:
                return String.class;
        }
    }

}
