package de.duehl.vocabulary.japanese.website.update.vocables;

import java.util.List;

import de.duehl.basics.datetime.date.ImmutualDate;
import de.duehl.basics.io.FileHelper;
import de.duehl.basics.text.Text;
import de.duehl.swing.ui.GuiTools;

import de.duehl.vocabulary.japanese.logic.VocabularyTrainerLogic;
import de.duehl.vocabulary.japanese.ui.VocabularyTrainerGui;
import de.duehl.vocabulary.japanese.ui.dialog.NewUpdateFoundDialog;
import de.duehl.vocabulary.japanese.website.update.vocables.parser.VocableVersionParser;

import static de.duehl.vocabulary.japanese.website.VocaluaryTrainerWebsiteConstants.*;
import static de.duehl.vocabulary.japanese.website.download.WebsiteFileContentDownloader.*;
import static de.duehl.vocabulary.japanese.website.update.UpdateTools.*;

/**
 * Diese Klasse kümmert sich um Updates der Vokabularien.
 *
 * @version 1.01     2024-10-05
 * @author Christian Dühl
 */

public class VocablesUpdater {

    /** Die Logik des Vokabel-Trainers. */
    private final VocabularyTrainerLogic logic;

    /** Die grafische Oberfläche des Vokabel-Trainers. */
    private final VocabularyTrainerGui gui;

    /**
     * Gibt an, dass die Prüfungen auf neue Versionen beim Start des Programms stattfinden.
     * Anderenfalls hat der Benutzer sie über das Update-Menü angestoßen.
     */
    private final boolean checkingNewVersionsOnStartUp;

    /**
     * Konstruktor.
     *
     * @param logic
     *            Die Logik des Vokabel-Trainers.
     * @param gui
     *            Die grafische Oberfläche des Vokabel-Trainers.
     * @param checkingNewVersionsOnStartUp
     *            Gibt an, dass die Prüfungen auf neue Versionen beim Start des Programms
     *            stattfinden. Anderenfalls hat der Benutzer sie über das Update-Menü angestoßen.
     */
    public VocablesUpdater(VocabularyTrainerLogic logic, VocabularyTrainerGui gui,
            boolean checkingNewVersionsOnStartUp) {
        this.logic = logic;
        this.gui = gui;
        this.checkingNewVersionsOnStartUp = checkingNewVersionsOnStartUp;
    }

    /** Prüft auf eine aktuellere Version der Vokabularien. */
    public void update() {
        String websiteVocableVersion = downloadVocableVersionFromServer();
        if (websiteVocableVersion.equals(NO_DOWNLOAD_SUCCESS)) {
            if (!checkingNewVersionsOnStartUp) {
                informUserAboutDownloadFailure("der Vokabeln", gui);
            }
        }
        else {
            checkVocabulariesVersionAgainstWebsiteVersion(websiteVocableVersion);
        }
    }

    private String downloadVocableVersionFromServer() {
        return downloadFileViaUrlFromServerAndGetContentOfFirstLine(
                NEWEST_VOCABLES_VERSION_ON_SERVER_URL);
    }

    private void checkVocabulariesVersionAgainstWebsiteVersion(String websiteVocableVersion) {
        String localVocableVersion = determineLocalVocableVersion();
        boolean localVocablesAreOlder = false;
        if (localVocableVersion.isEmpty()) {
            localVocablesAreOlder = true;
        }
        else {
            VocableVersionParser websiteVersionParser =
                    new VocableVersionParser(websiteVocableVersion);
            websiteVersionParser.parse();
            ImmutualDate websiteDate = websiteVersionParser.getDate();
            int websiteNumber = websiteVersionParser.getNumber();

            VocableVersionParser localVersionParser =
                    new VocableVersionParser(localVocableVersion);
            localVersionParser.parse();
            ImmutualDate localDate = localVersionParser.getDate();
            int localNumber = localVersionParser.getNumber();

            if (websiteDate.before(localDate)) {
                localVocablesAreOlder = false;
            }
            else if (websiteDate.after(localDate)) {
                localVocablesAreOlder = true;
            }
            else {
                localVocablesAreOlder = localNumber < websiteNumber;
            }
        }

        if (localVocablesAreOlder) {
            informUserAboutNewerVocableVesion(localVocableVersion, websiteVocableVersion);
        }
        else if (!checkingNewVersionsOnStartUp) {
            informUserAboutVocableVersionIsActual(localVocableVersion);
        }
    }

    private String determineLocalVocableVersion() {
        String localVocableDirectory = logic.getOptions().getVocabulariesPath();
        String versionFilename = FileHelper.concatPathes(localVocableDirectory,
                VOCABLES_VERSION_BARE_FILENAME_IN_LOCAL_VOCABLES_DIR);

        if (!FileHelper.isFile(versionFilename)) {
            Text.say("Keine Version der lokalen Vokabeln vorhanden, denn die lokale "
                    + "Datei '" + versionFilename + "' existiert nicht.");
            return "";
        }

        List<String> lines = FileHelper.readFileToList(versionFilename);
        if (lines.isEmpty()) {
            Text.say("Keine Version der lokalen Vokabeln vorhanden, denn die lokale "
                    + "Datei '" + versionFilename + "' ist leer.");
            return "";
        }
        else {
            return lines.get(0).trim();
        }
    }

    private void informUserAboutNewerVocableVesion(String localVocableVersion,
            String websiteVocableVersion) {
        String title = "Es gibt eine neuere Version der Vokabeln";
        String message = "Es gibt eine neuere Version der Vokabeln!\n\n"
                + "Aktuelle Version der Vokabeln: " + localVocableVersion + "\n"
                + "Neuere Version der Vokabeln: " + websiteVocableVersion;
        String beforeUrl = "Bitte besuchen sie die Seite";
        String url = SHORT_VOCBULARY_TRAINER_WEBPAGE_URL;
        String afterUrl = "und laden sie die neue Version herunter.";
        NewUpdateFoundDialog dialog = new NewUpdateFoundDialog(title, message, beforeUrl, url,
                afterUrl, gui.getLocation(), gui.getProgramImage());
        dialog.setVisible(true);
    }

    private void informUserAboutVocableVersionIsActual(String localVocableVersion) {
        String title = "Die Vokabeln ist aktuell";
        String message = "Die verwendete Version der Vokabeln " + localVocableVersion
                + " ist aktuell.";
        GuiTools.informUserInEdt(gui.getWindowAsComponent(), title, message);
    }

    /*
     * TODO
     *
     * Download aktuellerer Vokabularien:
     * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
     *
     * Bei den Vokabularien kann man die alten löschen (nach Rückfrage) und die neuen
     * auspacken, der Pfad ist in den Optionen und hier schon bekannt (siehe
     * determineLocalVocableVersion()).
     *
     * Danach muss das Programm neu gestartet werden. Das kann man sogar selbst veranlassen.
     *
     * Nach der Prüfung beendet sich das Programm nicht richtig.
     *
     *
     * 3) Download aktuellerer eigener Listen:
     *
     * Die Prüfung muss noch implementiert werden. -> checkNewOwnListsVersion()
     *
     * Bei den eigenen Listen ist der Pfad auch in den Optionen, man sollte wohl aber auch
     * fragen, wohin es gespeichert werden soll und dazu den Pfad benutzen. Siehe Export von
     * eigenen Listen.
     *
     * --------------------------------------------------------------------------------------------
     *
     * - Notizen zum Herunterladen binärer Dateien:
     *   https://stackoverflow.com/questions/10960409/how-do-i-save-a-file-downloaded-with-httpclient-into-a-specific-folder
     *   https://coderanch.com/t/676274/java/download-binary-file-web-server
     *   Das brauche ich, wenn ich über http die neue Version der Vokabeln oder des Programms
     *   herunterladen will.
     */

}
